/* Version 2 einer Wetterstation, die nun auf dem Wemos D1 mini basiert und 
 * aufgrund des damit vorhandenen Sleep-Modus wesentlich weniger Strom
 * benötigen sollte.
 * 
 * Es werden die Beleuchtungsstärke in lux, die Temperatur in °C, die Luft-
 * feuchtigkeit in % und der Luftdruck in hPa gemessen und an einen Server 
 * geschickt, damit sie sich mit dem Smartphone abrufen lassen.
 * 
 * Zusätzlich ist es möglich per Funk eine Teichstation einzubinden, deren Daten
 * ebenfalls an den Server geschickt werden.
 */

#include <Wire.h>                   // für alle I2C-Verbindungen (Bme280, BH1750)
#include <SPI.h>                    // für alle SPI-Verbindungen (nRF24L01)
#include <BH1750FVI.h>              // für den Lichtsensor BH1750
#include <Adafruit_Sensor.h>        // allgemeine Bibliothek für Sensoren, hier für den Bme280
#include <Adafruit_BME280.h>        // spezielle Bibliothek für den BME280
#include "ESP8266WiFi.h"            // WLAN vom ESP8266

// wenn die nächste Zeile einkommentiert wird, wird eine Verbindung zur Teichstation
// hergestellt, sonst nicht
//#define TEICHFUNK

// wenn die nächste Zeile einkommentiert wird, wird die Kommunikation über den Serial-Monitor 
// hergestellt, sonst nicht
//#define SERIALON

#define WLAN_SSID "DEINNETZWERKNAME"
#define WLAN_PASS "DEINPASSWORT"

const char* server = "www.voss89.de";
const String w_url = "/.../....php";
const String t_url = "/.../....php";
const int httpPort=80;

// Messintervall in ms
const uint64_t messintervall = 600000;
//time out loop count
const int timeout = 200;

//******* Variablen für die Daten der Wetterstation **************
uint16_t ev;                          // Beleuchtungsstaerke
uint32_t pres;                        // Luftdruck
uint32_t hum;                         // relative Luftfeuchtigkeit
int8_t temp;                          // Temperatur
float u_akku;


#ifdef TEICHFUNK
  #include "RF24.h"                             // für das Funk-Modul nRF24L01
  #include <nRF24L01.h>                         // für das Funk-Modul nRF24L01
  
  //***************** fuer das Funk-Modul *******************************
  byte addresses[][6] = {"1Node","2Node"};   // Festlegen der Adressen; dies kann jede Kombination von bis zu 6 Zahlen und Buchstaben sein
  bool radioNumber = 0;                      // Radio Nummer ist 0 oder 1; fuer Identifikationszwecke, denke ich
  RF24 radio(D3,D8);                           // Pin 6 des Nano ist mit Pin 3 (CE) des nRF24L01 verbunden
                                             // Pin 7 des Nano ist mit Pin 4 (CSN) des nRF24L01 verbunden

  //****************** Variablen für die Daten der Teichstation *************
  // die Variablen sind als Struktur zusammengefasst, damit sie leichter per Funk 
  // übertragen werden können; anscheinend muss es sich um float-Variablen handeln!
  struct dataStruct{
    float tempL=0;                             // Lufttemperatur
    float tempW0=0;                            // Wassertemperatur Oberflaeche
    float tempW1=0;                            // Wassertemperatur 1m tief
    float trueb=0;                            // Wassertruebung
    float wstand=0;                           // Wasserstand oder Teichtiefe
  } Teichdaten;
#endif

// Sensoren
BH1750FVI LightSensor;
Adafruit_BME280 bme;

void setup() {
  // Da das Programm Zeit braucht und das Messintervall immer genau abgerechnet
  // werden soll, wird die Zeit, die das Programm benötigt hat (inkl. Senden der
  // Daten) am Ende von der Schlafzeit abgezogen. Das hält außerdem beide 
  // Stationen synchron
  uint64_t startzeit = millis();
  
  WiFi.forceSleepWake();
  delay(1);
  WiFi.mode(WIFI_STA);              // ESP in Station-Mode

#ifdef SERIALON
  Serial.begin(115200); 
  Serial.println(); 
  Serial.print(F("Verbinde mit "));
  Serial.println(WLAN_SSID);
#endif

  WiFi.begin(WLAN_SSID, WLAN_PASS);
  int i = 0;
  for (i; i < timeout; i++)
  {
    if(WiFi.status() == WL_CONNECTED) break;
    delay(100);
  }
  if(i == timeout) deepSleep(messintervall);

#ifdef SERIALON
  Serial.println(F("IP address: ")); Serial.println(WiFi.localIP());
#endif


  LightSensor.begin();                          // BH1750 Start
  LightSensor.SetAddress(Device_Address_L);     // die I2C-Adresse wird auf 0x23 gesetzt (Alternativ 0x5c mit "_H" statt "_L")
                                                // der Addr-Pin des BH1750 muss  über einen 1kOhm Widerstand mit GND (also _L -> LOW) verbunden sein
  LightSensor.SetMode(Continuous_H_resolution_Mode);   // dies entspricht dem empfohlenen Arbeitsmodus; weitere gibt es im Datenblatt bzw der Bibliothek

  
                                                // Start des Bme280
  if (!bme.begin()) 
  {
#ifdef SERIALON
    Serial.println(F("BME280 nicht gestartet"));
#endif
  } 
  else 
  {
#ifdef SERIALON
    Serial.println(F("BME280 bereit"));
#endif
  }


  ev = LightSensor.GetLightIntensity();          // Lichtstaerke auslesen mit BH1750
  delay(200);  
  temp=bme.readTemperature();                    // Bme280 auslesen: Temperatur
  pres=bme.readPressure()/100.0F;                // Bme280 auslesen: Luftdruck
  hum=bme.readHumidity();                        // Bme280 auslesen: Luftfeuchtigkeit
  u_akku=analogRead(A0);
  u_akku*=0.0033;


#ifdef SERIALON
  Serial.print(F("Lichtstaerke: "));
  Serial.print(ev);
  Serial.println(F(" lx"));
  Serial.print(F("Temperatur: "));
  Serial.print(temp);
  Serial.println(F(" °C"));
  Serial.print(F("Luftdruck: "));
  Serial.print(pres);
  Serial.println(F(" hPa"));
  Serial.print(F("Luftfeuchtigkeit: "));
  Serial.print(hum);
  Serial.println(F(" %"));
  Serial.print(F("Spannung an Akku: "));
  Serial.print(u_akku);
  Serial.println(F("V"));

  Serial.println(F("Sende Daten an Server..."));
#endif

  
  String w_data = "t=" + String(temp) + "&hu=" + String(hum) + "&pre=" + String(pres) + "&li=" + String(ev) + "&ak=" + String(u_akku);
  httppost(w_url, w_data);


#ifdef TEICHFUNK
  delay(50);
  initiatepondradio();
  getponddata();

  String t_data = "tL=" + String(Teichdaten.tempL,0) // ",0" für null Nachkommastellen
                  + "&t0=" + String(Teichdaten.tempW0,0)
                  + "&t1=" + String(Teichdaten.tempW1,0) 
                  + "&tr=" + String(Teichdaten.trueb,0) 
                  + "&ws=" + String(Teichdaten.wstand,0);
  httppost(t_url, t_data);

  #ifdef SERIALON
    Serial.println(t_data);
  #endif
#endif

  // Berechnung der noch zu schlafenden Zeit (in Klammern steht die Wachzeit)
  uint64_t schlafzeit = messintervall - (millis() - startzeit); 
  #ifdef SERIALON
    unsigned long wachzeit = millis()-startzeit;
    Serial.print("Wachzeit: ");
    Serial.println(wachzeit);
  #endif
  
  deepSleep(schlafzeit);
}



/********************* Schlafmodus für weniger Stromverbrauch ******************/
void deepSleep(uint64_t schlafzeit)
{
  //https://github.com/esp8266/Arduino/issues/644
  WiFi.mode(WIFI_OFF);
  WiFi.forceSleepBegin();
  delay(1);
  ESP.deepSleep(schlafzeit * 1000/*, WAKE_RF_DISABLED*/);
}



/********************* Daten an Server schicken *******************************/
void httppost(String url, String data)
{
  if(WiFi.status()== WL_CONNECTED) //Check WiFi connection status
  {   
 
   WiFiClient http;                   //Declare object of class WiFiClient
   if (!http.connect(server, httpPort)) {
#ifdef SERIALON
    Serial.println(F("connection failed"));
#endif
    
    return;
  }
 
   http.println("POST " + url + " HTTP/1.1");
   http.println("Host: " + String(server));
   http.println("Accept: */*");
   http.println("Content-Type: application/x-www-form-urlencoded");
   http.print("Content-Length: ");
   http.println(data.length());
   http.println();
   http.print(data);

   delay(500);
   http.stop();
   
#ifdef SERIALON
   Serial.println(F("Data sent to Server"));
#endif

  }else{

#ifdef SERIALON
    Serial.println(F("Error in WiFi connection"));   
#endif
  
  }
}


#ifdef TEICHFUNK

/******************* Funkverbindung zum Teich aufbauen ***********************/
void initiatepondradio(void) {
  radio.begin();
  radio.setAutoAck(1);                          // Ensure autoACK is enabled
  radio.enableAckPayload();                     // Allow optional ack payloads
  radio.setRetries(10,15);                      // Smallest time between retries, max no. of retries
    
  // ************* NAECHSTEN BEFEHL SPAETER AUSKOMMENTIEREN
  radio.setPALevel(RF24_PA_LOW);                // beim Testen sind die Funkmodule nahe nebeneinander,
                                                // daher wird das PowerAmplifying Level heruntergeregelt; Default ist MAX
  radio.setDataRate(RF24_250KBPS);              // durch die geringste Datenrate in Kombination mit dem größten PA-Level bekommt man die größte Reichweite
  radio.setChannel(115);                          // wähle Channel, der nicht mit WLAN interferiert
  
                                                // Oeffnen der Kanaele mit der jeweils entgegengesetzten Adresse
  radio.openWritingPipe(addresses[0]);          // fuer Teichstation adresses[1]
  radio.openReadingPipe(1,addresses[1]);        // fuer Teichstation adressses[0]
}


/********************* Daten vom Teich empfangen *******************************/
void getponddata(void) {
  // Empfangen der Daten vom Teich
  radio.powerUp();
  radio.startListening();
  uint32_t waitingtime=0;
  while (!radio.available() && waitingtime<1100) 
  {
    delay(10);
    waitingtime++;
  } // tue nichts, solange keine Daten verfügbar sind oder ein Timeout (10s) vorliegt  
  if (radio.available())
  {
    while (radio.available()) 
    {  
      radio.read(&Teichdaten, sizeof(Teichdaten)); 
    }
    
    #ifdef SERIALON
    Serial.println(F("Daten erhalten"));
    #endif
  } 
  else 
  {
    #ifdef SERIALON
    Serial.println(F("Timeout beim Empfangen der Teichdaten!"));
    #endif
  }
  
  delay(1);
  radio.stopListening();
  radio.powerDown();
  delay(10);
}

#endif

void loop() 
{
  // put your main code here, to run repeatedly:

}
